package com.asseco.ce.xml.validation;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.net.URL;
import java.util.ArrayList;

import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;

public class Gui extends JFrame {

	private static final long serialVersionUID = 1812048172641082746L;

	private Container cont;

	private JLabel lblArea = new JLabel("Oblasť dát: ");
	
	private JComboBox moduleCombo;

	private JFilePicker filePicker = new JFilePicker("Cesta k súboru: ",
			"Hľadaj");

	private JButton btnValidate = new JButton("Validovať");

	private JLabel lblResult = new JLabel("");

	private JTextPane txtPane = new JTextPane();

	public Gui() {
		super("Nástroj na technickú validáciu XML súborov");

		// Custom icon
		URL urlIco = ClassLoader.getSystemResource("img/validate.png");
		if (urlIco != null){
			ImageIcon imgIco = new ImageIcon(urlIco);
			if (imgIco != null){
				setIconImage(new ImageIcon(ClassLoader
						.getSystemResource("img/validate.png")).getImage());
				setLayout(null);
				cont = getContentPane();
				cont.setLayout(null);
				cont.setBounds(0, 0, 700, 600);
				setSize(600, 500);
				setVisible(true);
			}
		}
		
		setResizable(false);
		
		// set up layout
		setLayout(new FlowLayout());

		// set up components
		filePicker.setMode(JFilePicker.MODE_OPEN);

		btnValidate.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				buttonValidateActionPerformed(event);
			}
		});

		// size of the screen
		Dimension scrSize = Toolkit.getDefaultToolkit().getScreenSize();
		this.setBounds(scrSize.width / 3, scrSize.height / 3, 600, 400);

		// add components to the frame
		JPanel pMain = new JPanel(new GridLayout(4, 1));
		pMain.setBounds(0, 0, 600, 400);
		add(pMain);

		JPanel pArea = new JPanel();
		pMain.add(pArea);

		pArea.add(lblArea);

		// Create the radio buttons.
		final DefaultComboBoxModel moduleNames = new DefaultComboBoxModel();
		for(String moduleName : getAvailableModules()){			
			moduleNames.addElement(moduleName/*.toUpperCase()*/);
		}		
		moduleCombo = new JComboBox(moduleNames);
		pArea.add(moduleCombo);

		pMain.add(filePicker, BorderLayout.NORTH);

		pMain.add(btnValidate);

		pMain.add(lblResult);

		//JPanel pText = new JPanel();
		//pText.setLayout(null);
		//pText.setPreferredSize(new Dimension(580, 100));
		txtPane.setBounds(0, 0, 580, 200);
		txtPane.setEditable(false);
		//pText.add(txtPane);
		//getContentPane().add(pText, BorderLayout.CENTER);
		
		JScrollPane scrlPane = new JScrollPane(txtPane);
		scrlPane.setPreferredSize(new Dimension(580, 200));
		scrlPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		//pMain.add(scrlPane);
		getContentPane().add(scrlPane, BorderLayout.CENTER);

		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent ev) {
				dispose();
				System.exit(0);
			}
		});
		this.setVisible(true);

		// OutOfMemory handling
		Thread
				.setDefaultUncaughtExceptionHandler(new Thread.UncaughtExceptionHandler() {
					public void uncaughtException(Thread t, Throwable e) {

						String eDetail = "";
						if (e.getClass().getName().equalsIgnoreCase(
								"java.lang.OutOfMemoryError")) {
							// Getting the runtime reference from system
							Runtime runtime = Runtime.getRuntime();
							eDetail = "(Out Of Memory - Heap:"
									+ Long.toString(runtime.maxMemory()) + ")";
						} else {
							eDetail = "(" + e.getClass().getName() + " : "
									+ e.getLocalizedMessage() + ")";
						}

						JOptionPane.showMessageDialog(null,
								"Interná chyba, prosím, kontaktujte správcu systému! "
										+ eDetail, "Chyba",
								JOptionPane.ERROR_MESSAGE);
					}
				});
	}

	/**
	 * handle click event of the Upload button
	 */
	private void buttonValidateActionPerformed(ActionEvent event) {
		lblResult.setText("");
		txtPane.setText("");

		moduleCombo.getSelectedItem();
		// Get area
	    String modul = (String)moduleCombo.getSelectedItem();
		if(modul == null || modul.equalsIgnoreCase("")){
			return;
		}

		String filePath = filePicker.getSelectedFilePath();

		if (filePath.equals("")) {
			JOptionPane.showMessageDialog(this,
					"Prosím zadajte súbor na validáciu!", "Chyba",
					JOptionPane.ERROR_MESSAGE);
			return;
		}

		// Validate
		try {
			ValidationResult vResult = XsdValidator.validate(modul, filePath);
			if (vResult.isValid()){
				lblResult.setText("XML je technicky validné. Biznis pravidlá sa nekontrolujú.");
				lblResult.setForeground(Color.BLACK);
			}else{
				lblResult.setText("XML je technicky nevalidné");
				lblResult.setForeground(Color.RED);
				txtPane.setText(vResult.getReason());
			}
			
		} catch (XsdValidationException e) {
			lblResult.setText("Chyba programe");
			lblResult.setForeground(Color.RED);
			txtPane.setText(e.getLocalizedMessage());
		}
		
	}
	
	private ArrayList<String> getAvailableModules(){
		
		ArrayList<String> modules = new ArrayList<String>();
		
		File xsdFolder = new File(XsdValidator.XSD_LOCATION);
		if (xsdFolder.exists()){
			for (File folder : xsdFolder.listFiles()){
				if (folder.isDirectory()){
					modules.add(folder.getName());
				}					
			}
		}
		
		return modules;
	}

}
