package com.asseco.ce.xml.validation;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class SchemeListFactory {
	
	private static final String NAMESPACE_XSD = "http://www.w3.org/2001/XMLSchema";
	
	public static ArrayList<SchemeBean> getSchemeList(String xsdLocation) throws XsdValidationException{
		
		ArrayList<SchemeBean> schemeList = new ArrayList<SchemeBean>();
		ArrayList<SchemeBean> sortedSchemeList = new ArrayList<SchemeBean>();
		
		//Nacist schemy
		File xsdDir = new File(xsdLocation);
		if(xsdDir == null){
			throw new XsdValidationException("Nepodarilo sa načitať Xsd priecinok ("+xsdLocation+")");
		}
		File[] files = xsdDir.listFiles();
		if(files != null){
			for (File file : files) {			
				//Pouze XSD soubory
				if (file.getName().toLowerCase().endsWith(".xsd")){
					schemeList.add(new SchemeBean(file.getName(),getImportedSchemes(file)));
				}			
			}			
		}else{
			throw new XsdValidationException("Nepodarilo sa načítať súbory v Xsd priecinku ("+xsdLocation+")");
		}
		
		//Usporadat - pred vlozenim schemy imported uz musi byt vlozene
		for(SchemeBean scheme : schemeList){
			ArrayList<SchemeBean> impotredSchemes = getImportedSchemes(sortedSchemeList, scheme);
			for (SchemeBean importedScheme : impotredSchemes){
				if(!sortedSchemeList.contains(importedScheme)){
					sortedSchemeList.add(importedScheme);
				}
			}
			if(!sortedSchemeList.contains(scheme)){
				sortedSchemeList.add(scheme);
			}
		}
		
		return sortedSchemeList;
	}
	
	private static ArrayList<SchemeBean> getImportedSchemes(ArrayList<SchemeBean> sortedSchemeList, SchemeBean scheme){
		ArrayList<SchemeBean> importsSchemeList = new ArrayList<SchemeBean>();
		
		for(SchemeBean imports : scheme.getImports()){
			ArrayList<SchemeBean> impotredSchemes = getImportedSchemes(sortedSchemeList, imports);
			for (SchemeBean importedScheme : impotredSchemes){
				if(!importsSchemeList.contains(importedScheme)){
					importsSchemeList.add(importedScheme);
				}
			}
			if(!importsSchemeList.contains(imports)){
				importsSchemeList.add(imports);
			}
		}
		
		return importsSchemeList;
	}
	
	private static ArrayList<SchemeBean> getImportedSchemes(File file) throws XsdValidationException{
		
		ArrayList<SchemeBean> schemes = new ArrayList<SchemeBean>();
		
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setNamespaceAware(true);

		DocumentBuilder builder;
		Document doc;
		try {
			builder = factory.newDocumentBuilder();
			doc = builder.parse(new FileInputStream(file));
			
			NodeList importNodes = doc.getElementsByTagNameNS(NAMESPACE_XSD, "import");
			
			for (int i = 0; importNodes != null && i < importNodes.getLength(); i++) {

				Node importNode = importNodes.item(i);
				NamedNodeMap nnm = importNode.getAttributes();
				String importSchema = nnm.getNamedItem("schemaLocation").getTextContent();
				
				File importFile = new File(file.getParentFile().getPath()+"/"+importSchema);
				
				schemes.add(new SchemeBean(importFile.getName(),getImportedSchemes(importFile)));
			}
			
		} catch (ParserConfigurationException e) {
			throw new XsdValidationException("Nepodarilo sa načitať Xsd schému ("+file.getName()+"): "+e.getLocalizedMessage());
		} catch (FileNotFoundException e) {
			throw new XsdValidationException("Nepodarilo sa načitať Xsd schému ("+file.getName()+"): "+e.getLocalizedMessage());
		} catch (SAXException e) {
			throw new XsdValidationException("Nepodarilo sa načitať Xsd schému ("+file.getName()+"): "+e.getLocalizedMessage());
		} catch (IOException e) {
			throw new XsdValidationException("Nepodarilo sa načitať Xsd schému ("+file.getName()+"): "+e.getLocalizedMessage());
		}
		
		return schemes;
		
	}	

}
