package com.asseco.ce.xml.validation;


import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public class XsdValidator {
	
	public static final String XSD_LOCATION = "xsd/";
	
	public static ValidationResult validate(String modul, String filePath) throws XsdValidationException {
		
		//=======Schema
		
		String modulePath = XSD_LOCATION + modul + "/";
		ArrayList<SchemeBean> xsdSchemeList = null;
		xsdSchemeList = SchemeListFactory.getSchemeList(modulePath);
		
		SchemaFactory factory = SchemaFactory
				.newInstance("http://www.w3.org/2001/XMLSchema");
		
		Schema schema = null;
		try {
			Source[] xsdSources = new Source[xsdSchemeList.size()];	
			int i = 0;
			for(SchemeBean xsdSchema : xsdSchemeList){
				xsdSources[i] = new StreamSource(new FileInputStream(modulePath + xsdSchema.getName()));
				i++;
			}
			schema = factory.newSchema(xsdSources);
		} catch (SAXException e) {
			throw new XsdValidationException("Nepodarilo sa načitať Xsd schémy: "+e.getLocalizedMessage());
		} catch (FileNotFoundException e) {
			throw new XsdValidationException("Nepodarilo sa načitať Xsd schémy: "+e.getLocalizedMessage());
		}
		if(schema == null){
			throw new XsdValidationException("Nepodarilo sa načitať Xsd schémy");
		}

		//======== File
		
		File file = new File(filePath);
		InputStream inputStream = null;
		try {
			inputStream = new FileInputStream(file);
		} catch (FileNotFoundException e1) {
			throw new XsdValidationException("Súbor sa nenašiel - " + e1.getLocalizedMessage());
		}
		
		if (inputStream == null){
			throw new XsdValidationException("Súbor sa nenašiel");
		}
		
		//Kontrola ZIP nebo XML
		if(filePath.toLowerCase().endsWith("zip")){
			
			HashMap<String,ByteArrayOutputStream> zMap = null;
			try {
				zMap = streamUnZip(inputStream);
			} catch (Exception e) {
				try {
					inputStream.close();
				} catch (Exception e1) {}
				throw new XsdValidationException("Nepodarilo sa načitať súbor: "+e.getLocalizedMessage());
			}
			
			try {
				inputStream.close();
			} catch (Exception e) {}
			
			if (zMap == null){
				throw new XsdValidationException("Nepodarilo sa načitať súbor");
			}
			if (zMap.size() != 1){
				return new ValidationResult(false,"Neočakávaný počet súborov v archíve: " + zMap.size());
			}	
			
			inputStream = new ByteArrayInputStream(((ByteArrayOutputStream) zMap.entrySet().iterator().next().getValue()).toByteArray());
			
		}else if (filePath.toLowerCase().endsWith("xml")){

		}else{
			//Necekany format
			try {
				inputStream.close();
			} catch (Exception e) {}
			return new ValidationResult(false,"Neočakávaný formát súboru");
		}
		
		Source source = new StreamSource(inputStream);
		Validator validator = schema.newValidator();
		
		try {
			validator.validate(source);
		} catch (SAXException e) {
			
			try {
				inputStream.close();
			} catch (Exception e1) {}
			
			if(e instanceof SAXParseException){
				return new ValidationResult(false,"(Riadok " + ((SAXParseException)e).getLineNumber() + ", Stĺpec " + ((SAXParseException)e).getColumnNumber() + "): " + e.getMessage());
			}else{
				return new ValidationResult(false,e.getMessage());
			}
		} catch (IOException e) {
			try {
				inputStream.close();
			} catch (Exception e1) {}
			return new ValidationResult(false,e.getMessage());
		}
		
		try {
			inputStream.close();
		} catch (Exception e) {}
		
		return new ValidationResult(true,"");
	}
	
	public static HashMap<String,ByteArrayOutputStream> streamUnZip(InputStream inputStream) throws IOException{

		HashMap<String,ByteArrayOutputStream> rMap = new HashMap<String,ByteArrayOutputStream>();
		
		//ZipInputStream zipIn = new ZipInputStream(inputStream);		
		ZipInputStream zipIn = new ZipInputStream(new BufferedInputStream(inputStream, 1024),Charset.forName("Cp437"));
				
		ZipEntry zipEntry = null;
		ByteArrayOutputStream baos = null;
		String fileName = null;

		while((zipEntry = zipIn.getNextEntry()) != null) {
			baos = new ByteArrayOutputStream();
			fileName = zipEntry.getName();
			for (int c = zipIn.read(); c != -1; c = zipIn.read()) {
				baos.write(c);
		    }
			rMap.put(fileName, baos);	        
			zipIn.closeEntry();
			baos.close();	
		}
		zipIn.close();
		
		return rMap;
	}
	
}
